@@ -1,20 +1,16 @@
 package POE::Component::IRC::Plugin::Connector;
 
+use strict;
+use warnings;
 use POE;
 use POE::Component::IRC 4.5;
 use POE::Component::IRC::Plugin qw( :ALL );
 
 sub new {
-  my ($package) = shift;
-
-  my $self = bless { @_ }, $package;
-
-  $self->{SESSION_ID} = POE::Session->create(
-	object_states => [
-	  $self => [ qw(_start _auto_ping _reconnect _shutdown _start_ping _stop_ping) ],
-	],
-	options => { trace => 0 },
-  )->ID();
+  my $package = shift;
+  my %parms = @_;
+  $parms{ lc $_ } = delete $parms{$_} for keys %parms;
+  my $self = bless \%parms, $package;
   return $self;
 }
 
@@ -22,10 +18,12 @@ sub PCI_register {
   my ($self,$irc) = splice @_, 0, 2;
 
   $self->{irc} = $irc;
-
-  if ( $irc->connected() ) {
-    $poe_kernel->post( $self->{SESSION_ID}, '_start_ping' );
-  }
+  $self->{SESSION_ID} = POE::Session->create(
+	object_states => [
+	  $self => [ qw(_start _auto_ping _reconnect _shutdown _start_ping _start_time_out _stop_ping _time_out) ],
+	],
+	options => { trace => 0 },
+  )->ID();
 
   $irc->plugin_register( $self, 'SERVER', qw(all) );
 
@@ -34,25 +32,26 @@ sub PCI_register {
 
 sub PCI_unregister {
   my ($self,$irc) = splice @_, 0, 2;
-
-  delete ( $self->{irc} );
-
+  delete $self->{irc};
   $poe_kernel->post( $self->{SESSION_ID} => '_shutdown' );
   $poe_kernel->refcount_decrement( $self->{SESSION_ID}, __PACKAGE__ );
-
   return 1;
 }
 
 sub S_connected {
   my ($self,$irc) = splice @_, 0, 2;
+  $poe_kernel->post( $self->{SESSION_ID}, '_start_time_out' );
+  return PCI_EAT_NONE;
+}
 
+sub S_001 {
+  my ($self,$irc) = splice @_, 0, 2;
   $poe_kernel->post( $self->{SESSION_ID}, '_start_ping' );
   return PCI_EAT_NONE;
 }
 
 sub S_disconnected {
   my ($self,$irc) = splice @_, 0, 2;
-
   $poe_kernel->post( $self->{SESSION_ID}, '_stop_ping' );
   $poe_kernel->post( $self->{SESSION_ID}, '_reconnect' );
   return PCI_EAT_NONE;
@@ -60,7 +59,6 @@ sub S_disconnected {
 
 sub S_error {
   my ($self,$irc) = splice @_, 0, 2;
-
   $poe_kernel->post( $self->{SESSION_ID}, '_stop_ping' );
   $poe_kernel->post( $self->{SESSION_ID}, '_reconnect' );
   return PCI_EAT_NONE;
@@ -68,7 +66,6 @@ sub S_error {
 
 sub S_socketerr {
   my ($self,$irc) = splice @_, 0, 2;
-
   $poe_kernel->post( $self->{SESSION_ID}, '_stop_ping' );
   $poe_kernel->post( $self->{SESSION_ID}, '_reconnect' );
   return PCI_EAT_NONE;
@@ -76,7 +73,7 @@ sub S_socketerr {
 
 sub S_pong {
   my ($self,$irc) = splice @_, 0, 2;
-  my ($reply) = ${ $_[0] };
+  my $reply = ${ $_[0] };
 
   if ( $reply and $reply =~ /^[0-9]+$/ ) {
 	$self->{lag} = time() - $reply;
@@ -99,36 +96,41 @@ sub _start {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
 
   $self->{SESSION_ID} = $_[SESSION]->ID();
-
   $kernel->refcount_increment( $self->{SESSION_ID}, __PACKAGE__ );
+  $kernel->yield( '_start_ping' ) if $self->{irc}->connected();
+  undef;
 }
 
 sub _start_ping {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
-
+  $kernel->delay( '_time_out' => undef );
   $kernel->delay( '_auto_ping' => $self->{delay} || 300 );
+  undef;
 }
 
 sub _auto_ping {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
 
-  if ( not $self->{seen_traffic} ) {
+  if ( !$self->{seen_traffic} ) {
      $self->{irc}->yield( 'ping' => time() );
   }
   $self->{seen_traffic} = 0;
   $kernel->yield( '_start_ping' );
+  undef;
 }
 
 sub _stop_ping {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
-
   $kernel->delay( '_auto_ping' => undef );
+  $kernel->delay( '_time_out' => undef );
+  undef;
 }
 
 sub _shutdown {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
 
   $kernel->yield( '_stop_ping' );
+  undef;
 }
 
 sub _reconnect {
@@ -139,10 +141,25 @@ sub _reconnect {
   } else {
 	$kernel->delay( '_reconnect' => 60 );
   }
+  undef;
+}
+
+sub _start_time_out {
+  my ($kernel,$self) = @_[KERNEL,OBJECT];
+
+  $kernel->delay( '_time_out' => $self->{timeout} || 60 );
+  undef;
+}
+
+sub _time_out {
+  my ($kernel,$self) = @_[KERNEL,OBJECT];
+
+  $self->{irc}->disconnect();
+  undef;
 }
 
 1;
-
+__END__
 =head1 NAME
 
 POE::Component::IRC::Plugin::Connector - A PoCo-IRC plugin that deals with the messy business of staying connected to an IRC server.
