@@ -57,24 +57,27 @@ sub S_socketerr {
 # Channel JOIN messages
 sub S_join {
   my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
   my $nick = ( split /!/, ${ $_[0] } )[0];
   my $userhost = ( split /!/, ${ $_[0] } )[1];
   my ($user,$host) = split(/\@/,$userhost);
   my $channel = ${ $_[1] };
+  my $uchan = u_irc $channel, $mapping;
+  my $unick = u_irc $nick, $mapping;
 
-  if ( u_irc ( $nick ) eq u_irc ( $self->{RealNick} ) ) {
-	delete $self->{STATE}->{Chans}->{ u_irc ( $channel ) };
-	$self->{CHANNEL_SYNCH}->{ u_irc ( $channel ) } = { MODE => 0, WHO => 0 };
-        $self->{STATE}->{Chans}->{ u_irc $channel } = { };
+  if ( $unick eq u_irc ( $self->nick_name(), $mapping ) ) {
+	delete $self->{STATE}->{Chans}->{ $uchan };
+	$self->{CHANNEL_SYNCH}->{ $uchan } = { MODE => 0, WHO => 0 };
+        $self->{STATE}->{Chans}->{ $uchan } = { };
         $self->yield ( 'who' => $channel );
         $self->yield ( 'mode' => $channel );
   } else {
         $self->yield ( 'who' => $nick );
-        $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{Nick} = $nick;
-        $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{User} = $user;
-        $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{Host} = $host;
-        $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{CHANS}->{ u_irc ( $channel ) } = '';
-        $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Nicks}->{ u_irc ( $nick ) } = '';
+        $self->{STATE}->{Nicks}->{ $unick }->{Nick} = $nick;
+        $self->{STATE}->{Nicks}->{ $unick }->{User} = $user;
+        $self->{STATE}->{Nicks}->{ $unick }->{Host} = $host;
+        $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan } = '';
+        $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $unick } = '';
   }
   return PCI_EAT_NONE;
 }
@@ -82,10 +85,11 @@ sub S_join {
 # Channel PART messages
 sub S_part {
   my ($self,$irc) = splice @_, 0, 2;
-  my $nick = u_irc ( ( split /!/, ${ $_[0] } )[0] );
-  my $channel = u_irc ${ $_[1] };
+  my $mapping = $irc->isupport('CASEMAPPING');
+  my $nick = u_irc ( ( split /!/, ${ $_[0] } )[0], $mapping );
+  my $channel = u_irc ${ $_[1] }, $mapping;
 
-  if ( $nick eq u_irc ( $self->nick_name() ) ) {
+  if ( $nick eq u_irc ( $self->nick_name(), $mapping ) ) {
         delete $self->{STATE}->{Nicks}->{ $nick }->{CHANS}->{ $channel };
         delete $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ $nick };
         foreach my $member ( keys %{ $self->{STATE}->{Chans}->{ $channel }->{Nicks} } ) {
@@ -108,16 +112,18 @@ sub S_part {
 # QUIT messages
 sub S_quit {
   my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
   my $nick = ( split /!/, ${ $_[0] } )[0];
   push @{ $_[2] }, [ $self->nick_channels( $nick ) ];
+  my $unick = u_irc $nick, $mapping;
 
-  if ( u_irc ( $nick ) eq u_irc ( $self->{RealNick} ) ) {
+  if ( $unick eq u_irc ( $self->nick_name(), $mapping ) ) {
         delete $self->{STATE};
   } else {
-        foreach my $channel ( keys %{ $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{CHANS} } ) {
-                delete $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ u_irc ( $nick ) };
+        foreach my $channel ( keys %{ $self->{STATE}->{Nicks}->{ $unick }->{CHANS} } ) {
+                delete $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ $unick };
         }
-        delete $self->{STATE}->{Nicks}->{ u_irc ( $nick ) };
+        delete $self->{STATE}->{Nicks}->{ $unick };
   }
   return PCI_EAT_NONE;
 }
@@ -125,24 +131,27 @@ sub S_quit {
 # Channel KICK messages
 sub S_kick {
   my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
   my $channel = ${ $_[1] };
   my $nick = ${ $_[2] };
+  my $unick = u_irc $nick, $mapping;
+  my $uchan = u_irc $channel, $mapping;
 
-  if ( u_irc ( $nick ) eq u_irc ( $self->{RealNick} ) ) {
-        delete $self->{STATE}->{Nicks}->{ u_irc $nick }->{CHANS}->{ u_irc $channel };
-        delete $self->{STATE}->{Chans}->{ u_irc $channel }->{Nicks}->{ u_irc $nick };
-        foreach my $member ( keys %{ $self->{STATE}->{Chans}->{ u_irc $channel }->{Nicks} } ) {
-           delete $self->{STATE}->{Nicks}->{ u_irc $member }->{CHANS}->{ u_irc $channel };
-           if ( scalar keys %{ $self->{STATE}->{Nicks}->{ u_irc $member }->{CHANS} } <= 0 ) {
-                delete $self->{STATE}->{Nicks}->{ u_irc $member };
+  if ( $unick eq u_irc ( $self->nick_name(), $mapping ) ) {
+        delete $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan };
+        delete $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $unick };
+        foreach my $member ( keys %{ $self->{STATE}->{Chans}->{ $uchan }->{Nicks} } ) {
+           delete $self->{STATE}->{Nicks}->{ $member }->{CHANS}->{ $uchan };
+           if ( scalar keys %{ $self->{STATE}->{Nicks}->{ $member }->{CHANS} } <= 0 ) {
+                delete $self->{STATE}->{Nicks}->{ $member };
            }
         }
-	delete $self->{STATE}->{Chans}->{ u_irc $channel };
+	delete $self->{STATE}->{Chans}->{ $uchan };
   } else {
-        delete $self->{STATE}->{Nicks}->{ u_irc $nick }->{CHANS}->{ u_irc $channel };
-        delete $self->{STATE}->{Chans}->{ u_irc $channel }->{Nicks}->{ u_irc $nick };
-        if ( scalar keys %{ $self->{STATE}->{Nicks}->{ u_irc $nick }->{CHANS} } <= 0 ) {
-                delete $self->{STATE}->{Nicks}->{ u_irc $nick };
+        delete $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan };
+        delete $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $unick };
+        if ( scalar keys %{ $self->{STATE}->{Nicks}->{ $unick }->{CHANS} } <= 0 ) {
+                delete $self->{STATE}->{Nicks}->{ $unick };
         }
   }
   return PCI_EAT_NONE;
@@ -151,25 +160,28 @@ sub S_kick {
 # NICK changes
 sub S_nick {
   my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
   my $nick = ( split /!/, ${ $_[0] } )[0];
   my $new = ${ $_[1] };
   push @{ $_[2] }, [ $self->nick_channels( $nick ) ];
+  my $unick = u_irc $nick, $mapping;
+  my $unew = u_irc $new, $mapping;
 
   if ( $nick eq $self->{RealNick} ) {
 	$self->{RealNick} = $new;
   }
 
-  if ( u_irc ( $nick ) eq u_irc ( $new ) ) {
+  if ( $unick eq $unew ) {
         # Case Change
-        $self->{STATE}->{Nicks}->{ u_irc $nick }->{Nick} = $new;
+        $self->{STATE}->{Nicks}->{ $unick }->{Nick} = $new;
   } else {
-        my $record = delete $self->{STATE}->{Nicks}->{ u_irc $nick };
+        my $record = delete $self->{STATE}->{Nicks}->{ $unick };
         $record->{Nick} = $new;
         foreach my $channel ( keys %{ $record->{CHANS} } ) {
-           $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ u_irc $new } = $record->{CHANS}->{ $channel };
-           delete $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ u_irc $nick };
+           $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ $unew } = $record->{CHANS}->{ $channel };
+           delete $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ $unick };
         }
-        $self->{STATE}->{Nicks}->{ u_irc $new } = $record;
+        $self->{STATE}->{Nicks}->{ $unew } = $record;
   }
   return PCI_EAT_NONE;
 }
@@ -177,13 +189,15 @@ sub S_nick {
 # Channel MODE
 sub S_mode {
   my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
   my $who = ${ $_[0] };
   my $channel = ${ $_[1] };
+  my $uchan = u_irc $channel, $mapping;
   pop @_;
   my @modes = map { ${ $_ } } @_[2 .. $#_];
 
   # Do nothing if it is UMODE
-  if ( u_irc ( $channel ) ne u_irc ( $self->{RealNick} ) ) {
+  if ( $uchan ne u_irc ( $self->{RealNick}, $mapping ) ) {
      my $parsed_mode = parse_mode_line( @modes );
      while ( my $mode = shift ( @{ $parsed_mode->{modes} } ) ) {
         my $arg;
@@ -191,17 +205,19 @@ sub S_mode {
         SWITCH: {
           if ( $mode =~ /\+([ohvaq])/ ) {
                 my $flag = $1;
-                unless ($self->{STATE}->{Nicks}->{ u_irc $arg }->{CHANS}->{ u_irc $channel }and $self->{STATE}->{Nicks}->{ u_irc $arg }->{CHANS}->{ u_irc $channel } =~ /$flag/) {
-                      $self->{STATE}->{Nicks}->{ u_irc $arg }->{CHANS}->{ u_irc $channel } .= $flag;
-                      $self->{STATE}->{Chans}->{ u_irc $channel }->{Nicks}->{ u_irc $arg } = $self->{STATE}->{Nicks}->{ u_irc $arg }->{CHANS}->{ u_irc $channel };
+		$arg = u_irc $arg, $mapping;
+                unless ($self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan }and $self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan } =~ /$flag/) {
+                      $self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan } .= $flag;
+                      $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $arg } = $self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan };
                 }
                 last SWITCH;
           }
           if ( $mode =~ /-([ohvaq])/ ) {
-                my ($flag) = $1;
-                if ($self->{STATE}->{Nicks}->{ u_irc ( $arg ) }->{CHANS}->{ u_irc ( $channel ) } =~ /$flag/) {
-                      $self->{STATE}->{Nicks}->{ u_irc ( $arg ) }->{CHANS}->{ u_irc ( $channel ) } =~ s/$flag//;
-                      $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Nicks}->{ u_irc ( $arg ) } = $self->{STATE}->{Nicks}->{ u_irc ( $arg ) }->{CHANS}->{ u_irc ( $channel ) };
+                my $flag = $1;
+		$arg = u_irc $arg, $mapping;
+                if ($self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan } =~ /$flag/) {
+                      $self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan } =~ s/$flag//;
+                      $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $arg } = $self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan };
                 }
                 last SWITCH;
           }
@@ -209,45 +225,45 @@ sub S_mode {
                 last SWITCH;
           }
           if ( $mode eq '+l' and defined ( $arg ) ) {
-                $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} .= 'l' unless ( $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} =~ /l/ );
-                $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{ChanLimit} = $arg;
+                $self->{STATE}->{Chans}->{ $uchan }->{Mode} .= 'l' unless ( $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ /l/ );
+                $self->{STATE}->{Chans}->{ $uchan }->{ChanLimit} = $arg;
                 last SWITCH;
           }
           if ( $mode eq '+k' and defined ( $arg ) ) {
-                $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} .= 'k' unless ( $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} =~ /k/ );
-                $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{ChanKey} = $arg;
+                $self->{STATE}->{Chans}->{ $uchan }->{Mode} .= 'k' unless ( $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ /k/ );
+                $self->{STATE}->{Chans}->{ $uchan }->{ChanKey} = $arg;
                 last SWITCH;
           }
           if ( $mode eq '-l' ) {
-                $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} =~ s/l//;
-                delete ( $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{ChanLimit} );
+                $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ s/l//;
+                delete $self->{STATE}->{Chans}->{ $uchan }->{ChanLimit};
                 last SWITCH;
           }
           if ( $mode eq '-k' ) {
-                $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} =~ s/k//;
-                delete ( $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{ChanKey} );
+                $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ s/k//;
+                delete $self->{STATE}->{Chans}->{ $uchan }->{ChanKey};
                 last SWITCH;
           }
           # Anything else doesn't have arguments so just adjust {Mode} as necessary.
           if ( $mode =~ /^\+(.)/ ) {
-                my ($flag) = $1;
-                $self->{STATE}->{Chans}->{ u_irc $channel }->{Mode} .= $flag unless $self->{STATE}->{Chans}->{ u_irc $channel }->{Mode} =~ /$flag/;
+                my $flag = $1;
+                $self->{STATE}->{Chans}->{ $uchan }->{Mode} .= $flag unless $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ /$flag/;
                 last SWITCH;
           }
           if ( $mode =~ /^-(.)/ ) {
-                my ($flag) = $1;
-                if ($self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} =~ /$flag/) {
-                      $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} =~ s/$flag//;
+                my $flag = $1;
+                if ($self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ /$flag/) {
+                      $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ s/$flag//;
                 }
                 last SWITCH;
           }
         }
      }
      # Lets make the channel mode nice
-     if ( $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} ) {
-        $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} = join('', sort {uc $a cmp uc $b} ( split( //, $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} ) ) );
+     if ( $self->{STATE}->{Chans}->{ $uchan }->{Mode} ) {
+        $self->{STATE}->{Chans}->{ $uchan }->{Mode} = join('', sort {uc $a cmp uc $b} ( split( //, $self->{STATE}->{Chans}->{ $uchan }->{Mode} ) ) );
      } else {
-        delete ( $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Mode} );
+        delete $self->{STATE}->{Chans}->{ $uchan }->{Mode};
      }
   }
   return PCI_EAT_NONE;
@@ -256,28 +272,31 @@ sub S_mode {
 # RPL_WHOREPLY
 sub S_352 {
   my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
   my ($first,$second) = split(/ :/,${ $_[1] } );
   my ($channel,$user,$host,$server,$nick,$status) = split(/ /,$first);
   my $real = substr($second,index($second," ")+1);
+  my $unick = u_irc $nick, $mapping;
+  my $uchan = u_irc $channel, $mapping;
 
-  $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{Nick} = $nick;
-  $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{User} = $user;
-  $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{Host} = $host;
-  $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{Real} = $real;
-  $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{Server} = $server;
+  $self->{STATE}->{Nicks}->{ $unick }->{Nick} = $nick;
+  $self->{STATE}->{Nicks}->{ $unick }->{User} = $user;
+  $self->{STATE}->{Nicks}->{ $unick }->{Host} = $host;
+  $self->{STATE}->{Nicks}->{ $unick }->{Real} = $real;
+  $self->{STATE}->{Nicks}->{ $unick }->{Server} = $server;
   if ( $channel ne '*' ) {
-    my ($whatever) = '';
+    my $whatever = '';
     if ( $status =~ /\@/ ) { $whatever = 'o'; }
     if ( $status =~ /\+/ ) { $whatever = 'v'; }
     if ( $status =~ /\%/ ) { $whatever = 'h'; }
     if ( $status =~ /\&/ ) { $whatever = 'a'; }
     if ( $status =~ /\~/ ) { $whatever = 'q'; }
-    $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{CHANS}->{ u_irc ( $channel ) } = $whatever;
-    $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Name} = $channel;
-    $self->{STATE}->{Chans}->{ u_irc ( $channel ) }->{Nicks}->{ u_irc ( $nick ) } = $whatever;
+    $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan } = $whatever;
+    $self->{STATE}->{Chans}->{ $uchan }->{Name} = $channel;
+    $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $unick } = $whatever;
   }
   if ( $status =~ /\*/ ) {
-    $self->{STATE}->{Nicks}->{ u_irc ( $nick ) }->{IRCop} = 1;
+    $self->{STATE}->{Nicks}->{ $unick }->{IRCop} = 1;
   }
   return PCI_EAT_NONE;
 }
@@ -285,13 +304,15 @@ sub S_352 {
 #RPL_ENDOFWHO
 sub S_315 {
   my ($self,$irc) = splice @_, 0, 2;
-  my ($channel) = ( split / :/, ${ $_[1] } )[0];
+  my $mapping = $irc->isupport('CASEMAPPING');
+  my $channel = ( split / :/, ${ $_[1] } )[0];
+  my $uchan = u_irc $channel, $mapping;
 
   # If it begins with #, &, + or ! its a channel apparently. RFC2812.
   if ( $channel =~ /^[\x23\x2B\x21\x26]/ ) {
     $self->_channel_sync_who($channel);
     if ( $self->_channel_sync($channel) ) {
-	delete ( $self->{CHANNEL_SYNCH}->{ u_irc ( $channel ) } );
+	delete $self->{CHANNEL_SYNCH}->{ $uchan };
 	$self->_send_event( 'irc_chan_sync', $channel );
     }
   # Otherwise we assume its a nickname
@@ -304,32 +325,34 @@ sub S_315 {
 # RPL_CHANNELMODEIS
 sub S_324 {
   my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
   my @args = split / /, ${ $_[1] };
   my $channel = shift @args;
+  my $uchan = u_irc $channel, $mapping;
 
   my $parsed_mode = parse_mode_line( @args );
   while ( my $mode = shift ( @{ $parsed_mode->{modes} } ) ) {
         $mode =~ s/\+//;
         my $arg;
         $arg = shift @{ $parsed_mode->{args} } if $mode =~ /[kl]/;
-	if ( $self->{STATE}->{Chans}->{ u_irc $channel }->{Mode} ) {
-          $self->{STATE}->{Chans}->{ u_irc $channel }->{Mode} .= $mode unless $self->{STATE}->{Chans}->{ u_irc $channel }->{Mode} =~ /$mode/;
+	if ( $self->{STATE}->{Chans}->{ $uchan }->{Mode} ) {
+          $self->{STATE}->{Chans}->{ $uchan }->{Mode} .= $mode unless $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ /$mode/;
 	} else {
-	  $self->{STATE}->{Chans}->{ u_irc $channel }->{Mode} = $mode;
+	  $self->{STATE}->{Chans}->{ $uchan }->{Mode} = $mode;
 	}
         if ( $mode eq 'l' and defined ( $arg ) ) {
-           $self->{STATE}->{Chans}->{ u_irc $channel }->{ChanLimit} = $arg;
+           $self->{STATE}->{Chans}->{ $uchan }->{ChanLimit} = $arg;
         }
         if ( $mode eq 'k' and defined ( $arg ) ) {
-           $self->{STATE}->{Chans}->{ u_irc $channel }->{ChanKey} = $arg;
+           $self->{STATE}->{Chans}->{ $uchan }->{ChanKey} = $arg;
         }
   }
-  if ( $self->{STATE}->{Chans}->{ u_irc $channel }->{Mode} ) {
-        $self->{STATE}->{Chans}->{ u_irc $channel }->{Mode} = join('', sort {uc $a cmp uc $b} split //, $self->{STATE}->{Chans}->{ u_irc $channel }->{Mode} );
+  if ( $self->{STATE}->{Chans}->{ $uchan }->{Mode} ) {
+        $self->{STATE}->{Chans}->{ $uchan }->{Mode} = join('', sort {uc $a cmp uc $b} split //, $self->{STATE}->{Chans}->{ $uchan }->{Mode} );
   }
   $self->_channel_sync_mode($channel);
   if ( $self->_channel_sync($channel) ) {
-	delete $self->{CHANNEL_SYNCH}->{ u_irc $channel };
+	delete $self->{CHANNEL_SYNCH}->{ $uchan };
 	$self->_send_event( 'irc_chan_sync', $channel );
   }
   return PCI_EAT_NONE;
@@ -341,11 +364,10 @@ sub S_324 {
 
 sub _channel_sync {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
 
-  unless ( $self->_channel_exists($channel) ) {
-	return 0;
-  }
+  return 0 unless $self->_channel_exists($channel);
 
   if ( defined ( $self->{CHANNEL_SYNCH}->{ $channel } ) ) {
 	if ( $self->{CHANNEL_SYNCH}->{ $channel }->{MODE} and $self->{CHANNEL_SYNCH}->{ $channel }->{WHO} ) {
@@ -357,11 +379,10 @@ sub _channel_sync {
 
 sub _channel_sync_mode {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
 
-  unless ( $self->_channel_exists($channel) ) {
-	return 0;
-  }
+  return 0 unless $self->_channel_exists($channel);
 
   if ( defined ( $self->{CHANNEL_SYNCH}->{ $channel } ) ) {
 	$self->{CHANNEL_SYNCH}->{ $channel }->{MODE} = 1;
@@ -372,11 +393,10 @@ sub _channel_sync_mode {
 
 sub _channel_sync_who {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
 
-  unless ( $self->_channel_exists($channel) ) {
-	return 0;
-  }
+  return 0 unless $self->_channel_exists($channel);
 
   if ( defined ( $self->{CHANNEL_SYNCH}->{ $channel } ) ) {
 	$self->{CHANNEL_SYNCH}->{ $channel }->{WHO} = 1;
@@ -387,33 +407,28 @@ sub _channel_sync_who {
 
 sub _nick_exists {
   my $self = shift;
-  my $nick = u_irc ( $_[0] ) || return 0;
-
-  if ( defined ( $self->{STATE}->{Nicks}->{ $nick } ) ) {
-	return 1;
-  }
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $nick = u_irc ( $_[0], $mapping ) || return 0;
+  return 1 if defined $self->{STATE}->{Nicks}->{ $nick };
   return 0;
 }
 
 sub _channel_exists {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
-
-  if ( defined ( $self->{STATE}->{Chans}->{ $channel } ) ) {
-	return 1;
-  }
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  return 1 if defined $self->{STATE}->{Chans}->{ $channel };
   return 0;
 }
 
 sub _nick_has_channel_mode {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
-  my $nick = u_irc ( $_[1] ) || return 0;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $nick = u_irc ( $_[1], $mapping ) || return 0;
   my $flag = ( split //, $_[2] )[0] || return 0;
 
-  unless ( $self->is_channel_member($channel,$nick) ) {
-	return 0;
-  }
+  return 0 unless $self->is_channel_member($channel,$nick);
 
   if ( $self->{STATE}->{Nicks}->{ $nick }->{CHANS}->{ $channel } =~ /$flag/ ) {
 	return 1;
@@ -424,8 +439,9 @@ sub _nick_has_channel_mode {
 # Returns all the channels that the bot is on with an indication of whether it has operator, halfop or voice.
 sub channels {
   my $self = shift;
+  my $mapping = $self->isupport('CASEMAPPING');
   my %result;
-  my $realnick = u_irc $self->{RealNick};
+  my $realnick = u_irc $self->{RealNick}, $mapping;
 
   if ( $self->_nick_exists($realnick) ) {
 	foreach my $channel ( keys %{ $self->{STATE}->{Nicks}->{ $realnick }->{CHANS} } ) {
@@ -440,51 +456,48 @@ sub nicks {
   my @result;
 
   foreach my $nick ( keys %{ $self->{STATE}->{Nicks} } ) {
-	push ( @result, $self->{STATE}->{Nicks}->{ $nick }->{Nick} );
+	push @result, $self->{STATE}->{Nicks}->{ $nick }->{Nick};
   }
   return @result;
 }
 
 sub nick_info {
   my $self = shift;
-  my $nick = u_irc ( $_[0] ) || return undef;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $nick = u_irc ( $_[0], $mapping ) || return;
 
-  unless ( $self->_nick_exists($nick) ) {
-	return undef;
-  }
+  return unless $self->_nick_exists($nick);
 
-  my ($record) = $self->{STATE}->{Nicks}->{ $nick };
+  my $record = $self->{STATE}->{Nicks}->{ $nick };
 
-  my (%result) = %{ $record };
+  my %result = %{ $record };
 
   $result{Userhost} = $result{User} . '@' . $result{Host};
 
-  delete ( $result{'CHANS'} );
+  delete $result{'CHANS'};
 
   return \%result;
 }
 
 sub nick_long_form {
   my $self = shift;
-  my $nick = u_irc ( $_[0] ) || return undef;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $nick = u_irc ( $_[0], $mapping ) || return;
 
-  unless ( $self->_nick_exists($nick) ) {
-	return undef;
-  }
+  return unless $self->_nick_exists($nick);
 
-  my ($record) = $self->{STATE}->{Nicks}->{ $nick };
+  my $record = $self->{STATE}->{Nicks}->{ $nick };
 
   return $record->{Nick} . '!' . $record->{User} . '@' . $record->{Host};
 }
 
 sub nick_channels {
   my $self = shift;
-  my $nick = u_irc ( $_[0] ) || return ();
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $nick = u_irc ( $_[0], $mapping ) || return;
   my @result;
 
-  unless ( $self->_nick_exists($nick) ) {
-	return @result;
-  }
+  return unless $self->_nick_exists($nick);
 
   foreach my $channel ( keys %{ $self->{STATE}->{Nicks}->{ $nick }->{CHANS} } ) {
 	push ( @result, $self->{STATE}->{Chans}->{ $channel }->{Name} );
@@ -494,15 +507,14 @@ sub nick_channels {
 
 sub channel_list {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return undef;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return;
   my @result;
 
-  unless ( $self->_channel_exists($channel) ) {
-	return undef;
-  }
+  return unless $self->_channel_exists($channel);
 
   foreach my $nick ( keys %{ $self->{STATE}->{Chans}->{ $channel }->{Nicks} } ) {
-	push( @result, $self->{STATE}->{Nicks}->{ $nick }->{Nick} );
+	push @result, $self->{STATE}->{Nicks}->{ $nick }->{Nick};
   }
 
   return @result;
@@ -510,28 +522,22 @@ sub channel_list {
 
 sub is_operator {
   my $self = shift;
-  my $nick = u_irc ( $_[0] ) || return 0;
-
-  unless ( $self->_nick_exists($nick) ) {
-	return 0;
-  }
-
-  if ( $self->{STATE}->{Nicks}->{ $nick }->{IRCop} ) {
-	return 1;
-  }
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $nick = u_irc ( $_[0], $mapping ) || return;
+  return unless $self->_nick_exists($nick);
+  return 1 if $self->{STATE}->{Nicks}->{ $nick }->{IRCop};
   return 0;
 }
 
 sub is_channel_mode_set {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
   my $mode = ( split //, $_[1] )[0] || return 0;
 
   $mode =~ s/[^A-Za-z]//g;
 
-  unless ( $self->_channel_exists($channel) or $mode ) {
-	return 0;
-  }
+  return unless $self->_channel_exists($channel) or $mode;
 
   if ( defined ( $self->{STATE}->{Chans}->{ $channel }->{Mode} ) and $self->{STATE}->{Chans}->{ $channel }->{Mode} =~ /$mode/ ) {
 	return 1;
@@ -541,11 +547,10 @@ sub is_channel_mode_set {
 
 sub channel_limit {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return undef;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
 
-  unless ( $self->_channel_exists($channel) ) {
-	return undef;
-  }
+  return unless $self->_channel_exists($channel);
 
   if ( $self->is_channel_mode_set($channel,'l') and defined ( $self->{STATE}->{Chans}->{ $channel }->{ChanLimit} ) ) {
 	return $self->{STATE}->{Chans}->{ $channel }->{ChanLimit};
@@ -555,11 +560,10 @@ sub channel_limit {
 
 sub channel_key {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return undef;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
 
-  unless ( $self->_channel_exists($channel) ) {
-	return undef;
-  }
+  return unless $self->_channel_exists($channel);
 
   if ( $self->is_channel_mode_set($channel,'k') and defined ( $self->{STATE}->{Chans}->{ $channel }->{ChanKey} ) ) {
 	return $self->{STATE}->{Chans}->{ $channel }->{ChanKey};
@@ -569,11 +573,10 @@ sub channel_key {
 
 sub channel_modes {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return undef;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
 
-  unless ( $self->_channel_exists($channel) ) {
-	return undef;
-  }
+  return unless $self->_channel_exists($channel);
 
   if ( defined ( $self->{STATE}->{Chans}->{ $channel }->{Mode} ) ) {
 	return $self->{STATE}->{Chans}->{ $channel }->{Mode};
@@ -583,86 +586,70 @@ sub channel_modes {
 
 sub is_channel_member {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
-  my $nick = u_irc ( $_[1] ) || return 0;
-
-  unless ( $self->_channel_exists($channel) and $self->_nick_exists($nick) ) {
-	return 0;
-  }
-
-  if ( defined ( $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ $nick } ) ) {
-	return 1;
-  }
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  return unless $self->_channel_exists($channel) and $self->_nick_exists($nick);
+  return 1 if defined $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ $nick };
   return 0;
 }
 
 sub is_channel_operator {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
-  my $nick = u_irc ( $_[1] ) || return 0;
-
-  unless ( $self->_nick_has_channel_mode($channel,$nick,'o') ) {
-	return 0;
-  }
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  return 0 unless $self->_nick_has_channel_mode($channel,$nick,'o');
   return 1;
 }
 
 sub has_channel_voice {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
-  my $nick = u_irc ( $_[1] ) || return 0;
-
-  unless ( $self->_nick_has_channel_mode($channel,$nick,'v') ) {
-	return 0;
-  }
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  return 0 unless $self->_nick_has_channel_mode($channel,$nick,'v');
   return 1;
 }
 
 sub is_channel_halfop {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
-  my $nick = u_irc ( $_[1] ) || return 0;
-
-  unless ( $self->_nick_has_channel_mode($channel,$nick,'h') ) {
-	return 0;
-  }
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  return 0 unless $self->_nick_has_channel_mode($channel,$nick,'h');
   return 1;
 }
 
 sub is_channel_owner {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
-  my $nick = u_irc ( $_[1] ) || return 0;
-
-  unless ( $self->_nick_has_channel_mode($channel,$nick,'q') ) {
-        return 0;
-  }
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  return 0 unless $self->_nick_has_channel_mode($channel,$nick,'q');
   return 1;
 }
 
 sub is_channel_admin {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return 0;
-  my $nick = u_irc ( $_[1] ) || return 0;
-
-  unless ( $self->_nick_has_channel_mode($channel,$nick,'a') ) {
-        return 0;
-  }
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  return 0 unless $self->_nick_has_channel_mode($channel,$nick,'a');
   return 1;
 }
 
 sub ban_mask {
   my $self = shift;
-  my $channel = u_irc ( $_[0] ) || return undef;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return undef;
   my $mask = parse_ban_mask ( $_[1] ) || return undef;
   my @result;
 
-  unless ( $self->_channel_exists($channel) ) {
-	return @result;
-  }
+  return unless $self->_channel_exists($channel);
 
   # Convert the mask from IRC to regex.
-  $mask = u_irc ( $mask );
+  $mask = u_irc ( $mask, $mapping );
   $mask = quotemeta $mask;
   $mask =~ s/\\\*/[\x01-\xFF]{0,}/g;
   $mask =~ s/\\\?/[\x01-\xFF]{1,1}/g;
